/**
* HTML-Coding-Aufgaben verwalten
*/
function HTMLTask(
    arrTasks,
    parentNode      // Elternelement, in dem sich der Editor befinden soll
) {
    this.parentNode = parentNode;
    this.tasks = arrTasks;
    this.levellist = new Levelliste(parentNode);
    this.levellist.count = arrTasks.length;
    this.levellist.onchange = function(level) { self._startLevel(level); };
    this.nodeTask = HTMLEditor.createHtml('div', parentNode, { 'class': 'auftrag' });
    this.editor = new HTMLEditor(parentNode);
    var self = this;
    this.editor.onchange = function(html) { self.check(html); };
    this.connect();
}

/**
* Kontrollieren, ob HTML richtig ist
* @return undefined
*/
HTMLTask.prototype.check = function(
    html
) {
    var parser = new DOMParser();
    var doc = parser.parseFromString(html, "text/html");
    if ((typeof this.tasks[this.levellist.active].check == 'function') && this.tasks[this.levellist.active].check.call(this, doc)) {
        if (this.levellist.done <= this.levellist.active)
            this.levellist.done++;
        if (this.levellist.active < this.levellist.count - 1)
            this.levellist.active++;
        if (this.canSave())
            this.saveScore(this.levellist.done, 0, this.levellist.count, (this.levellist.done == this.levellist.count) ? 'completed' : 'incomplete');
        if (this.levellist.done == this.levellist.count)
            this.showMsg('Aufgabe erfolgreich abgeschlossen!', 'msg_success');
    }
}

/**
* Level Nummer level starten
* @return undefined
*/
HTMLTask.prototype._startLevel = function(
    level
) {
    this.nodeTask.innerHTML = this.tasks[level].task ? this.tasks[level].task : '';
    if (this.tasks[level].html)
        this.editor.value = this.tasks[level].html;
}

/**
* Versuchen, eine Verbindung zur Lernplattform aufzubauen
* @return undefined
*/
HTMLTask.prototype.connect = function() {
    if (typeof LMSLink == 'function') {
        this.lmslink = new LMSLink();
        this.lmslink.start();
        if (this.lmslink && this.lmslink.ready) {
            if (this.lmslink.preferenceAudio == -1)
                this.muted = true;
        }
    }
    if (!this.lmslink || !this.lmslink.ready) {
        this.showMsg('Verbindungsaufnahme zur Lernplattform fehlgeschlagen - Fortschritte können nicht gespeichert werden', 'msg_warn', true);
        this.levellist.active = 0;
    } else {
        if (this.getDone()) {
            if (this.lmslink.lessonStatus == 'failed')
                this.showMsg('Aufgabe wurde engültig nicht geschafft', 'msg_failed', true);
            else
                this.showMsg('Aufgabe wurde bereits früher erfolgreich erledigt', 'msg_success', true);
            this.levellist.done = this.tasks.length;
            this.levellist.active = 0;
        } else {
            this.levellist.done = Math.min(this.tasks.length, this.lmslink.score);
            this.levellist.active = Math.min(this.levellist.done, this.levellist.count - 1);
        }
    }
}

// Genutzte Instanz von LMSLink
HTMLTask.prototype.lmslink = null;

// Aktive Meldung
HTMLTask.prototype.nodeMsg = null;

/**
* Eine Meldung anzeigen
* @return undefined
*/
HTMLTask.prototype.showMsg = function(
    strText,        // Textinhalt der Meldung
    strClass        // Klassenname der Meldung
) {
    var node = HTMLEditor.createHtml('p', null, { "class": strClass });
    HTMLEditor.createHtml('span', HTMLEditor.createHtml('span', node), null, strText);
    if (this.nodeMsg)
        this.nodeMsg.parentNode.replaceChild(node, this.nodeMsg);
    else
        this.parentNode.insertBefore(node, this.parentNode.firstChild);
    this.nodeMsg = node;
}

/**
* Punktestand und Status speichern
* @return undefined
*/
HTMLTask.prototype.saveScore = function(
    scoreRaw,       // Erreichter Punktestand
    scoreMin,       // Minimal erreichbarer Punktestand
    scoreMax,       // Maximal erreichbarer Punktestand
    status          // Optional: lessonStatus
) {
    if (this.canSave()) {
        this.lmslink.lessonStatus = status ? status : 'incomplete';
        this.lmslink.scoreMin = scoreMin;
        this.lmslink.scoreMax = scoreMax;
        this.lmslink.score = scoreRaw;
    }
    this.reportTime();
}

/**
* Ermittelt, ob die Aufgabe bereits erfolgreich gelöst wurde
* @return Boolean       Aufgabe bereits gelöst?
*/
HTMLTask.prototype.getDone = function() {
    return (this.lmslink && this.lmslink.ready && (['completed', 'passed', 'failed'].indexOf(this.lmslink.lessonStatus) != -1));
}

/**
* Prüft, ob Veränderungen am Lernpaket gespeichert werden sollen
* @return Boolean       Kann und soll gespeichert werden?
*/
HTMLTask.prototype.canSave = function() {
    if (this._canSave === undefined)
        this._canSave = (this.lmslink && this.lmslink.ready && (this.lmslink.lessonMode == 'normal'));
    return (this._canSave && (['completed', 'passed', 'failed'].indexOf(this.lmslink.lessonStatus) == -1));
}

/**
* Den Zwischenstand an Bearbeitungszeit an die Lernplattform melden
* @return undefined
*/
HTMLTask.prototype.reportTime = function() {
    if (this.lmslink && this.lmslink.ready)
        this.lmslink.sessionTime += 0;
}
