/**
* Klasse, die eine Levelliste verwaltet
*/
function Levelliste(
    nodeParent,     // DOM-Element, als dessen Kind die Levelliste angezeigt werden soll
    nodeAfter       // Optional: Kind von nodeParent, vor dem die Levelliste eingefügt werden soll
) {
    this._establishDOM(nodeParent, nodeAfter);
}

// Bei Levelwechsel oder -neustart aufzurufende Methode der Form function("Wert von this.active")
Levelliste.prototype.onchange = null;

// Bezeichnung für Level
Levelliste.prototype.strLevelName = 'Level ';

/**
* Levelliste im DOM anlegen
* @return undefined
*/
Levelliste.prototype._establishDOM = function(
    nodeParent,     // DOM-Element, als dessen Kind die Levelliste angezeigt werden soll
    nodeAfter       // Optional: Kind von nodeParent, vor dem die Levelliste eingefügt werden soll
) {
    if (nodeAfter)
        nodeParent = nodeAfter.parentNode;
    var doc = nodeParent.ownerDocument;
    var node = doc.createElement('nav');
    node.className = 'levellist';
    node.appendChild(doc.createElement('ul'));
    node = node.firstChild;
    node.className = 'levellist';
    Object.defineProperty(this, 'node', { value: node, writeable: false, enumerable: true });
    nodeParent.insertBefore(node.parentNode, nodeAfter);
}

// UL-Element, in dem die möglichen Level aufgelistet werden
Levelliste.prototype.node = null;

// Müssen die Level der Reihenfolge nach gelöst werden?
Levelliste.prototype.requirePreviousDone = true;

// Anzahl der angezeigten Level
Object.defineProperty(Levelliste.prototype, 'count', {
    get() {
        return (this.node ? this.node.childNodes.length : 0);
    },
    set(v) {
        if (!this.node)
            return;
        while (this.count < v)
            this._addLevelBtn();
        while (this.count > v)
            this._removeLevelBtn();
    },
    enumerable: true
});

// Anzahl der bereits erfolgreich absolvierten Level
Object.defineProperty(Levelliste.prototype, 'done', {
    get() {
        if (!this.node)
            return undefined;
        var n = 0;
        for (var node = this.node.firstChild; node; node = node.nextSibling) {
            if (node.firstChild.classList.contains('done'))
                ++n;
            else
                break;
        }
        return n;
    },
    set(v) {
        if (!this.node)
            return;
        v = Math.max(0, Math.floor(v));
        while (this.count < v)
            this._addLevelBtn();
        var old = this.done;
        for (old; old < v; ++old)
            this.node.childNodes[old].firstChild.classList.add('done');
        for (old; old > v; --old)
            this.node.childNodes[old - 1].firstChild.classList.remove('done');
    },
    enumerable: true
});

// Nummer (beginnend bei 0) des aktuellen Levels
Object.defineProperty(Levelliste.prototype, 'active', {
    get() {
        if (!this.node)
            return undefined;
        for (var n = 0; n < this.node.childNodes.length; ++n) {
            if (this.node.childNodes[n].firstChild.classList.contains('active'))
                return n;
        }
    },
    set(v) {
        if (!this.node)
            return;
        if (this.requirePreviousDone && (v > this.done))
            return;
        var old = this.active;
        if (Number.isFinite(old))
            this.node.childNodes[old].firstChild.classList.remove('active');
        v = Math.min(this.count - 1, Math.max(0, Math.floor(v)));
        if (Number.isFinite(v))
            this.node.childNodes[v].firstChild.classList.add('active');
        if (typeof this.onchange == 'function')
            this.onchange.call(this, v);
    },
    enumerable: true
});

/**
* Eine neue Schaltfläche für ein Level erstellen
* @return undefined
*/
Levelliste.prototype._addLevelBtn = function() {
    var doc = this.node.ownerDocument
    var nodeLi = doc.createElement('li');
    var nodeA = doc.createElement('a');
    nodeA.href = '#';
    var self = this;
    var n = this.node.childNodes.length;
    nodeA.appendChild(doc.createTextNode(this.strLevelName + (n + 1)));
    nodeA.addEventListener('click', function(ev) {
        ev.preventDefault();
        ev.stopPropagation();
        self.active = n;
    });
    nodeLi.appendChild(nodeA);
    this.node.appendChild(nodeLi);
}

/**
* Schaltfläche für höchstes Level wieder entfernen
* @return undefined
*/
Levelliste.prototype._removeLevelBtn = function() {
    if (!this.node.firstChild)
        return;
    this.node.removeChild(this.node.lastChild);
}
